@@ -1,4 +1,8 @@
-import torch
+# Copyright (c) Microsoft Corporation.
+# SPDX-License-Identifier: Apache-2.0
+
+# DeepSpeed Team
+
 from .builder import CUDAOpBuilder
 
 
@@ -16,20 +20,16 @@ class QuantizerBuilder(CUDAOpBuilder):
     def sources(self):
         return [
             'csrc/quantization/pt_binding.cpp',
-            'csrc/quantization/quantizer.cu',
+            'csrc/quantization/fake_quantizer.cu',
+            'csrc/quantization/quantize.cu',
+            'csrc/quantization/quantize_intX.cu',
+            'csrc/quantization/dequantize.cu',
+            'csrc/quantization/swizzled_quantize.cu',
+            'csrc/quantization/quant_reduce.cu',
         ]
 
     def include_paths(self):
         return ['csrc/includes']
 
-    def nvcc_args(self):
-        args = [
-            '-O3',
-            '--use_fast_math',
-            '-std=c++14',
-            '-U__CUDA_NO_HALF_OPERATORS__',
-            '-U__CUDA_NO_HALF_CONVERSIONS__',
-            '-U__CUDA_NO_HALF2_OPERATORS__'
-        ]
-
-        return args + self.compute_capability_args()
+    def extra_ldflags(self):
+        return ['-lcurand']
